//<c>   Copyright 1988-1999 by Gerry J. Danen; all rights reserved.

/*
<t>     Locate files across the file system

        Will scan any and all hard drives and network drives attached to your
        machine for whatever file you are looking for.  Wildcards are, of
        course, accepted.  The program will tell you the drive and and path of
        all found instances.

        Renamed to WHEREP, the program outputs just the pathname.
*/
#include"gd_tools.h"
#include"gd_extrn.h"


#define pgVERS_ "V4.31"
#define pgTITL_ "Locate Files"
#define pgCOPR_ "1988"


#if     (! LARGE_MODEL)
  #error LARGE MEMORY MODEL REQUIRED
#endif


union   fileDateUnion                           // global unions
{
        ms_date_t dt ;
        unsigned pdt ;
}       fldate ;
union   fileTimeUnion
{
        ms_time_t tm ;
        unsigned ptm ;
}       fltime ;


BOOL    p_switch = FALSE, ynProcessSingle=FALSE, ynStartCurrent=FALSE ;
int     iFormatType = 0 ;                       // output format
ULONG   found = 0, matched_files_bytes = 0 ;
INT16   sys_day, sys_mth, sys_yr ;
UCHAR   searchstr[FS_LEN] = "" ;                // file spec to search for
char    *months[] =
{
    "Jan", "Feb", "Mar", "Apr", "May", "Jun",
    "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
};


//      Prototypes
void    match_drives(UCHAR drives[],UCHAR fspec[]);
void    match_files(void);


void    program_usage( void )
{
    logon(1);
    fprintf( stderr, "\n\tUsage: %s [d:]filename.ext [parameters]\n"
            "\n\tWildcards (* and ?) allowed, no path names (e.g., NO c:\\*.txt)\n"
            "\n\tSet GD.DRIVES to restrict drives to search\n",
            get_global_program_name() );
    if ( st_eq( get_global_program_name(), "WhereP" ) )
        fprintf( stderr, "\n\tRename WHEREP to WHERE for standard output" );
    else
        fprintf( stderr, "\n\tRename %s to WHEREP for alternate output", get_global_program_name() );
    fprintf( stderr, "\n\n\tOptional [parameter]: /.  = start at current directory\n"
                     "\t\t\t      /f1 = format 1: directory + filename\n"
                     "\t\t\t      /f2 = format 2: filename + directory\n"
                     "\t\t\t      /f3 = format 3: filename + info + directory\n"
                     "\t\t\t      /f4 = format 4: info like format 3 in .CSV format\n\n" );
    exit( BAD_EXIT );
}


void    match_drives( UCHAR drives[], UCHAR fspec[] )
{
    int         i ;

    if ( ynStartCurrent )
    {
        ynProcessSingle = TRUE ;
        return ;
    }
    _strlwr( fspec );
    if ( fspec[1] != DRIV_SEP_CH )              // no drive included
        return ;
    if ( fspec[0] < 'a' || fspec[0] > 'z' )     // not a drive letter
        return ;
    ynProcessSingle = TRUE ;
    for ( i = 0 ; i < 26 ; i++ )                // reset drives to search
        drives[i] = 0 ;
    drives[ fspec[0] - 'a' ] = 1 ;              // just this one
    fspec[0] = fspec[1] = SPACE ;               // remove drive
    st_trim( fspec );
}


void    match_files( void )
/*
        This procedure searches for files that match the SEARCHSTR.  Once a
        match is found, its name and other information is printed on the
        screen.  The date and time as well as the path are all displayed.
        This procedure is recursive.
*/
{
    char        curdir[FS_LEN], temp[3], AorP, sDate[15], sTimeSize[25] ;
    int         done, attrib, month, day, year, hour, minute ;
    struct find_t dta ;
    BOOL        isRoot ;
    ULONG       ymd, hms ;

    _getcwd( curdir, sizeof(curdir) );
    if ( p_switch )
        _strlwr( curdir );

    if ( (strlen(curdir) == 3) && (curdir[1] == DRIV_SEP_CH) )
        isRoot = TRUE ;
    else
        isRoot = FALSE ;

    attrib = REGULAR_FILES ;                    // all except directories, and volid
    done = _dos_findfirst( searchstr, attrib, &dta );
    if ( ! p_switch )
        if ( ! done )
            printf( "\n%s\n", curdir );

    while ( ! done )
    {
        fltime.ptm = dta.wr_time ;
        hour   = fltime.tm.hh ;
        minute = fltime.tm.mm ;
        fldate.pdt = dta.wr_date ;
        day    = fldate.dt.d ;
        month  = fldate.dt.m ;
        year   = fldate.dt.y + 1980 ;

        AorP = (char) ((hour < 12) ? 'a' : 'p');
        hour %= 12 ;
        if ( hour == 0 )
            hour = 12 ;

        sprintf( sTimeSize, "%02d:%02d%cm  %10lu  ", hour, minute, AorP, dta.size );
        if ( month == 0 )
            sprintf( sDate, "%11s", " " );      // error
        else
        if ( sys_day == day && sys_mth == month && sys_yr == year )
            sprintf( sDate, "%-11s", "Today" );
        else
            sprintf( sDate, "%02d-%3s-%04d", day, months[month - 1], year );

        _strlwr( dta.name );
        if ( p_switch )
        {
            switch( iFormatType )
            {
                case 0 :
                    printf( "%s%s%s\n", curdir, (isRoot ? "" : "\\"), dta.name );
                    break ;
                case 1 :
                    printf( "%-67.67s%s\n", curdir, dta.name );
                    break ;
                case 2 :
                    printf( "%-12.12s %s\n", dta.name, curdir );
                    break ;
                case 3 :
                    printf( "%-12.12s %s  %s%s\n", dta.name, sDate, sTimeSize, curdir );
                    break ;
                case 4 :
                    sprintf( sDate, "%02d-%3s-%04d", day, months[month - 1], year );
                    hms = (fltime.tm.hh * 10000L) + (fltime.tm.mm * 100L) + fltime.tm.ss ;
                    ymd = (year * 10000L) + (month * 100L) + day ;
                    printf( "\"%s\",\"%s\",\"%02d:%02d%cm\",%lu,%lu,%lu,\"%s\"\n",
                            dta.name, sDate, hour, minute, AorP, dta.size, ymd, hms, curdir );
                    break ;
            }
        }
        else
            printf( "\t\t%-12s\t%s  %s\n", dta.name, sDate, sTimeSize );

        found ++ ;
        matched_files_bytes += dta.size ;
        done = _dos_findnext( &dta );
    }

    attrib = _A_SUBDIR+_A_SYSTEM+_A_HIDDEN ;
    done = _dos_findfirst( "*.*", attrib, &dta );

    while ( ! done )
    {
        strncpy(temp,dta.name, 2);
        if ( ((dta.attrib & _A_SUBDIR) == _A_SUBDIR) && (temp[0] != '.') )
        {
            if ( ! isRoot )
                strcat( curdir, "\\" );
            strcat( curdir, dta.name );
            _chdir( curdir );
            match_files();                      // recursive call
            _chdir( ".." );
            _getcwd( curdir, sizeof(curdir) );
        }
        done = _dos_findnext( &dta );
    }
}


void    main( int argc, char *argv[] )
/*
        The main section of this utility does several interesting things.  One
        of these is the creation of a table of valid drives.  It assumes that a
        system can have 26 drives (one per letter of the alphabet) and proceeds
        to check each drive.
*/
{
    UCHAR       availdrvs[26] ;
    char        origdir[FS_LEN], rootdir[FS_LEN], tempdir[FS_LEN] ;
    unsigned    numofdrives ;
    int         i ;
    INT32       sys_ymd ;

    tt_init( SHAREWARE_VERSION, 0, pgVERS_, pgTITL_, pgCOPR_ );
    logon(0);

    if ( st_eq( get_global_program_name(), "WhereP" ) )
        p_switch = TRUE ;

//~~~
//  atexit( ... );                              // save/restore paths

    if ( argc < 2 )
        program_usage();

    argc -= 1 ;
    LOOP( i, 1, argc, 1 )                       // process switches first
    {
        ST_CPY( tempdir, argv[i] );
        if ( tempdir[0] == '/' )
        {
            if ( st_eq( &tempdir[1], "." ) )
                ynStartCurrent = TRUE ;
            if ( st_eq( &tempdir[1], "f1" ) )
                iFormatType = 1 ;
            if ( st_eq( &tempdir[1], "f2" ) )
                iFormatType = 2 ;
            if ( st_eq( &tempdir[1], "f3" ) )
                 iFormatType = 3 ;
            if ( st_eq( &tempdir[1], "f4" ) )
                 iFormatType = 4 ;
        }
    }
    LOOP( i, 1, argc, 1 )                       // process parameters
    {
        ST_CPY( tempdir, argv[i] );
        if ( tempdir[0] != '/' )
            ST_CPY( searchstr, tempdir );
    }

    if ( ! strlen( searchstr ) )
        program_usage();

    if ( ynStartCurrent )                       // remove drive letter
    {                                           //  for current "/."
        _strlwr( searchstr );
        if ( searchstr[1] == DRIV_SEP_CH &&
             searchstr[0] NOT_LESS 'a' && searchstr[0] NOT_GR 'z' )
        {
            searchstr[0] = searchstr[1] = SPACE ;
            st_trim( searchstr );
        }
    }

    if ( iFormatType > 0 )
        p_switch = TRUE ;

    dt_getsystemdate( &sys_ymd, &sys_yr, &sys_mth, &sys_day );

    _getcwd( origdir, sizeof(origdir) );        // get current directory
    ds_avail_drives( availdrvs );
    availdrvs[0] = availdrvs[1] = 0 ;           // bypass a: and b: by default
    match_drives( availdrvs, searchstr );       // check for drive in filespec

    if ( ! ynProcessSingle )
    {
        int ndrives = 0 ;

        for ( i = 0 ; i < 26 ; i++ )            // count drives to process
        {
            if ( availdrvs[i] == 1 )
                ndrives += 1 ;
        }
        if ( ndrives > 0 )                      // are there _any_ drives?
        {
            fprintf( stderr, "Checking drive%s:", ndrives==1 ? "" : "s" );
            for ( i = 0 ; i < 26 ; i++ )        // A: to Z:
            {
                if ( availdrvs[i] == 1 )
                {
                    fprintf( stderr, " %c", (char) ('A' + i) );
                }
            }
            fprintf( stderr, "\n" );
        }
    }

    if ( ynStartCurrent )                       // start at current directory
    {
        match_files();
    }
    else
    {
        for ( i = 0 ; i < 26 ; i++ )            // A: to Z:
        {
            if ( availdrvs[i] == 1 )
            {
                rootdir[0] = (char) ('A' + i) ; // form a drive letter
                rootdir[1] = DRIV_SEP_CH ;      //  followed by a colon
                rootdir[2] = PATH_SEP_CH ;      //  and a backslash.
                rootdir[3] = 0 ;
                if ( ! ynProcessSingle )
                    fprintf( stderr, "\nDrive %c: ", rootdir[0] );
                _dos_setdrive( i+1, &numofdrives );
                _getcwd( tempdir, sizeof(tempdir) );
                _chdir( rootdir );              // switch to root
                match_files();
                _chdir( tempdir );
            }
        }
    }

//~~~
//  atexit( ... );                              // save/restore paths
    _dos_setdrive( origdir[0] - 64, &numofdrives );
    _chdir( origdir );                          // back to original

    if ( ! p_switch )
    {
        if ( found )
        {
            fprintf( stderr, "\n\t\t%ld match%s, %lu bytes\n",
                     found, (found==1L ? "" : "es"),
                     matched_files_bytes );
        }
        else
        {
            strupr ( searchstr );
            fprintf( stderr, "\nNo files match %s\n", searchstr );
        }
    }
    exit( GOOD_EXIT );
}
